@@ -1,6 +1,6 @@
-#-*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 
-# Copyright 2008-2013 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2008-2015 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@
 import re
 from calculate.lib.format.samba import samba
 
+
 class compiz(samba):
     """Класс для обработки конфигурационного файла типа compiz
 
@@ -24,15 +25,15 @@ class compiz(samba):
     _comment = "#"
     configName = "compiz"
     configVersion = "0.1"
-    reHeader = re.compile("^[\t ]*\[[^\[\]]+\].*\n",re.M)
-    reBody = re.compile(".+",re.M|re.S)
-    reComment = re.compile("\s*%s.*"%(_comment))
+    reHeader = re.compile("^[\t ]*\[[^\[\]]+\].*\n", re.M)
+    reBody = re.compile(".+", re.M | re.S)
+    reComment = re.compile("\s*%s.*" % _comment)
     reSeparator = re.compile("\s*=\s*")
     sepFields = "\n"
     reSepFields = re.compile(sepFields)
 
-    def __init__(self,text):
-        samba.__init__(self,text)
+    def __init__(self, text):
+        samba.__init__(self, text)
 
     def join(self, compizObj):
         """Объединяем конфигурации"""
