@@ -1,6 +1,6 @@
-#-*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 
-# Copyright 2014 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2014-2015 Calculate Ltd. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -17,13 +17,18 @@ from functools import wraps
 
 from itertools import tee
 from contextlib import contextmanager
-import json
+try:
+    import json as json_module
+except ImportError:
+    json_module = None
+
 
 class SavableIterator:
     """
     Итератор с возможность сохранять и восстанавливать состояние
     """
-    def __init__(self,seq):
+
+    def __init__(self, seq):
         self.seq = iter(seq)
         self.back = None
 
@@ -42,6 +47,7 @@ class SavableIterator:
     def next(self):
         return self.seq.next()
 
+
 @contextmanager
 def ignore(exception):
     try:
@@ -49,6 +55,7 @@ def ignore(exception):
     except exception:
         pass
 
+
 class AddonError(Exception):
     """
     Исключение с добавочным сообщением
@@ -64,7 +71,8 @@ class FalseObject(object):
     """
     Объект-заглушка при любом сравнении возвращает False
     """
-    def __lt__(self,other):
+
+    def __lt__(self, other):
         return False
 
     def __nonzero__(self):
@@ -83,14 +91,15 @@ class Sizes(object):
     T = TiB = tibibyte = G * 1024
     Tb = terabyte = Gb * 1000
 
-    def __getattr__(self,name):
+    def __getattr__(self, name):
         if name.startswith('from_'):
-            return lambda x:x*getattr(Sizes,name[5:])
+            return lambda x: x * getattr(Sizes, name[5:])
         elif name.startswith('to_'):
-            return lambda x:x/getattr(Sizes,name[3:])
+            return lambda x: x / getattr(Sizes, name[3:])
         else:
             raise AttributeError
 
+
 def imap_regexp(re_compiled, l, whole=False):
     """
     Обработать список регулярным выражением и вернуть полученные группы
@@ -101,10 +110,12 @@ def imap_regexp(re_compiled, l, whole=False):
         retfunc = lambda x: x.groups()
     return (retfunc(x) for x in (re_compiled.search(x) for x in l) if x)
 
+
 def cached(each_instance=False):
     """
     Кэширование результата
     """
+
     def cache_wrapper(func):
         value = {}
 
@@ -114,11 +125,13 @@ def cached(each_instance=False):
                     value[args[0]] = func(*args, **kwargs)
                 return value[args[0]]
             else:
-                if not None in value:
+                if None not in value:
                     value[None] = func(*args, **kwargs)
                 return value[None]
+
         return wrapper
-    if each_instance in (True,False):
+
+    if each_instance in (True, False):
         return cache_wrapper
     else:
         return cache_wrapper(each_instance)
@@ -127,44 +140,59 @@ def cached(each_instance=False):
 class Singleton(type):
     _instances = {}
 
-    def __call__(cls, *args, **kwargs):
+    def __call__(cls, *args, **kw):
         if cls not in cls._instances:
-            cls._instances[cls] = super(Singleton, cls).__call__(*args,
-                                                                 **kwargs)
+            create_obj = getattr(super(Singleton, cls), '__call__')
+            cls._instances[cls] = create_obj(*args, **kw)
         return cls._instances[cls]
 
 
+class SingletonParam(type):
+    def __init__(cls, name, bases, dicts):
+        super(SingletonParam, cls).__init__(name, bases, dicts)
+        cls.instance = {}
+
+    def __call__(cls, *args, **kw):
+        keyarg = args[0] if args else ""
+        if keyarg not in cls.instance:
+            create_obj = getattr(super(SingletonParam, cls), '__call__')
+            cls.instance[keyarg] = create_obj(*args, **kw)
+        return cls.instance[keyarg]
+
+
 class ClassifyingIterator(object):
     """
     Классифицирующий итератор
     """
 
+
 def classificate(iterator):
     """
     Классифицировать элементы по признаку (first первый last последний)
     """
+
     class Mark:
         def __init__(self, first=False, last=False):
             self.first = first
             self.last = last
 
         def __repr__(self):
-            return "Mark(first=%s,last=%s)"%(self.first,self.last)
+            return "Mark(first=%s,last=%s)" % (self.first, self.last)
 
     iterator = iter(iterator)
     obj = iterator.next()
     try:
         obj_next = iterator.next()
-        yield Mark(first=True,last=False), obj
+        yield Mark(first=True, last=False), obj
         try:
             while True:
                 obj = obj_next
                 obj_next = iterator.next()
                 yield Mark(), obj
         except StopIteration:
-            yield Mark(first=False,last=True), obj
+            yield Mark(first=False, last=True), obj
     except StopIteration:
-        yield Mark(first=True,last=True), obj
+        yield Mark(first=True, last=True), obj
 
 
 def debug(prefix="DEBUG {name}({args},{kw})={ret}"):
@@ -174,7 +202,9 @@ def debug(prefix="DEBUG {name}({args},{kw})={ret}"):
             ret = f(*args, **kw)
             print prefix.format(name=f.func_name, args=args, kw=kw, ret=ret)
             return ret
+
         return wrapper
+
     return debug_decor
 
 
@@ -207,6 +237,7 @@ def classproperty(func):
 
     return ClassPropertyDescriptor(func)
 
+
 import os
 import fcntl
 from os import path
@@ -282,7 +313,7 @@ class Locker(object):
         """ Activated when used in the with statement.
             Should automatically acquire a lock to be used in the with block.
         """
-        if self.lockf is not None:
+        if self.lockf is None:
             self.acquire()
         return self
 
@@ -326,3 +357,19 @@ def max_default(iterable, key=lambda x: x, default=None):
         return max(iterable, key=key)
     except ValueError:
         return default
+
+def traverse(o, tree_types=(list, tuple)):
+    """
+    Вернуть последовательно все элемены списка, "распаковов" встроенные
+
+    [1, 2, [1,2,[4,6,7]], 7] -> (1,2,1,2,4,6,7,7)
+    :param o:
+    :param tree_types: типы, которые будут распакованы
+    :return:
+    """
+    if isinstance(o, tree_types):
+        for value in o:
+            for subvalue in traverse(value, tree_types):
+                yield subvalue
+    else:
+        yield o
